# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/TimeToLiveApi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.ai/h2o/featurestore/api/v1/TimeToLiveApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"L\n\x06Online\"B\n\x12TimeToLiveInterval\x12\x08\n\x04\x44\x41YS\x10\x00\x12\n\n\x06MONTHS\x10\x01\x12\t\n\x05HOURS\x10\x02\x12\x0b\n\x07MINUTES\x10\x03\"K\n\x07Offline\"@\n\x12TimeToLiveInterval\x12\t\n\x05YEARS\x10\x00\x12\x08\n\x04\x44\x41YS\x10\x01\x12\n\n\x06MONTHS\x10\x02\x12\t\n\x05HOURS\x10\x03\"\xdf\x01\n\nTimeToLive\x12\x13\n\x0bttl_offline\x18\x01 \x01(\r\x12T\n\x14ttl_offline_interval\x18\x02 \x01(\x0e\x32\x36.ai.h2o.featurestore.api.v1.Offline.TimeToLiveInterval\x12\x12\n\nttl_online\x18\x03 \x01(\r\x12R\n\x13ttl_online_interval\x18\x04 \x01(\x0e\x32\x35.ai.h2o.featurestore.api.v1.Online.TimeToLiveIntervalB\x14\x42\x12TimeToLiveApiProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.TimeToLiveApi_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\022TimeToLiveApiProto'
  _globals['_ONLINE']._serialized_start=78
  _globals['_ONLINE']._serialized_end=154
  _globals['_ONLINE_TIMETOLIVEINTERVAL']._serialized_start=88
  _globals['_ONLINE_TIMETOLIVEINTERVAL']._serialized_end=154
  _globals['_OFFLINE']._serialized_start=156
  _globals['_OFFLINE']._serialized_end=231
  _globals['_OFFLINE_TIMETOLIVEINTERVAL']._serialized_start=167
  _globals['_OFFLINE_TIMETOLIVEINTERVAL']._serialized_end=231
  _globals['_TIMETOLIVE']._serialized_start=234
  _globals['_TIMETOLIVE']._serialized_end=457
# @@protoc_insertion_point(module_scope)
