# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/DashboardApi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-ai/h2o/featurestore/api/v1/DashboardApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"a\n\x11\x44\x61shboardResponse\x12\x16\n\x0eproject_number\x18\x01 \x01(\x04\x12\x1b\n\x13\x66\x65\x61ture_sets_number\x18\x02 \x01(\x04\x12\x17\n\x0f\x66\x65\x61tures_number\x18\x03 \x01(\x04\x42\x13\x42\x11\x44\x61shboardApiProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.DashboardApi_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\021DashboardApiProto'
  _globals['_DASHBOARDRESPONSE']._serialized_start=77
  _globals['_DASHBOARDRESPONSE']._serialized_end=174
# @@protoc_insertion_point(module_scope)
